"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const common_tags_1 = require("common-tags");
exports.generate = {
    signature: 'api-key generate <name>',
    description: 'Generate a new API key with the given name',
    help: common_tags_1.stripIndent `
		This command generates a new API key for the current user, with the given
		name. The key will be logged to the console.

		This key can be used to log into the CLI using 'resin login --token <key>',
		or to authenticate requests to the API with an 'Authorization: Bearer <key>' header.

		Examples:

			$ resin api-key generate "Jenkins Key"
	`,
    action(params, _options, done) {
        return __awaiter(this, void 0, void 0, function* () {
            const resin = (yield Promise.resolve().then(() => require('resin-sdk'))).fromSharedOptions();
            resin.models.apiKey
                .create(params.name)
                .then(key => {
                console.log(common_tags_1.stripIndent `
					Registered api key '${params.name}':

					${key}

					This key will not be shown again, so please save it now.
				`);
            })
                .finally(done);
        });
    },
};
//# sourceMappingURL=api-key.js.map