"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Bluebird = require("bluebird");
const common_tags_1 = require("common-tags");
exports.join = {
    signature: 'join [deviceIp]',
    description: 'Promote a local device running unmanaged resinOS to join a resin.io application',
    help: common_tags_1.stripIndent `
		Examples:

			$ resin join
			$ resin join resin.local
			$ resin join resin.local --application MyApp
			$ resin join 192.168.1.25
			$ resin join 192.168.1.25 --application MyApp
	`,
    options: [
        {
            signature: 'application',
            parameter: 'application',
            alias: 'a',
            description: 'The name of the application the device should join',
        },
    ],
    primary: true,
    action(params, options, done) {
        return __awaiter(this, void 0, void 0, function* () {
            const resin = yield Promise.resolve().then(() => require('resin-sdk'));
            const Logger = yield Promise.resolve().then(() => require('../utils/logger'));
            const promote = yield Promise.resolve().then(() => require('../utils/promote'));
            const sdk = resin.fromSharedOptions();
            const logger = new Logger();
            return Bluebird.try(() => {
                return promote.join(logger, sdk, params.deviceIp, options.application);
            }).nodeify(done);
        });
    },
};
//# sourceMappingURL=join.js.map