// Generated by CoffeeScript 1.12.7

/*
Copyright 2016 Resin.io

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */
var Promise, bodyParser, createServer, express, getContext, path, resin, utils;

express = require('express');

path = require('path');

bodyParser = require('body-parser');

Promise = require('bluebird');

resin = require('resin-sdk-preconfigured');

utils = require('./utils');

createServer = function(arg) {
  var app, isDev, port, ref, server;
  ref = arg != null ? arg : {}, port = ref.port, isDev = ref.isDev;
  app = express();
  app.use(bodyParser.urlencoded({
    extended: true
  }));
  app.set('view engine', 'ejs');
  app.set('views', path.join(__dirname, 'pages'));
  if (isDev) {
    app.use(express["static"](path.join(__dirname, 'pages', 'static')));
  }
  server = app.listen(port);
  return {
    app: app,
    server: server
  };
};


/**
 * @summary Await for token
 * @function
 * @protected
 *
 * @param {Object} options - options
 * @param {String} options.path - callback path
 * @param {Number} options.port - http port
 *
 * @example
 * server.awaitForToken
 * 	path: '/auth'
 * 	port: 9001
 * .then (token) ->
 *   console.log(token)
 */

exports.awaitForToken = function(options) {
  var app, ref, server;
  ref = createServer({
    port: options.port
  }), app = ref.app, server = ref.server;
  return new Promise(function(resolve, reject) {
    var closeServer, renderAndDone;
    closeServer = function(errorMessage, successPayload) {
      return server.close(function() {
        if (errorMessage) {
          reject(new Error(errorMessage));
          return;
        }
        return resolve(successPayload);
      });
    };
    renderAndDone = function(arg) {
      var errorMessage, request, response, statusCode, token, viewName;
      request = arg.request, response = arg.response, viewName = arg.viewName, errorMessage = arg.errorMessage, statusCode = arg.statusCode, token = arg.token;
      return getContext(viewName).then(function(context) {
        response.status(statusCode || 200).render(viewName, context);
        request.connection.destroy();
        return closeServer(errorMessage, token);
      });
    };
    app.post(options.path, function(request, response) {
      var ref1, token;
      token = (ref1 = request.body.token) != null ? ref1.trim() : void 0;
      return Promise["try"](function() {
        if (!token) {
          throw new Error('No token');
        }
        return utils.loginIfTokenValid(token);
      }).tap(function(loggedIn) {
        if (!loggedIn) {
          throw new Error('Invalid token');
        }
      }).then(function() {
        return renderAndDone({
          request: request,
          response: response,
          viewName: 'success',
          token: token
        });
      })["catch"](function(error) {
        return renderAndDone({
          request: request,
          response: response,
          viewName: 'error',
          statusCode: 401,
          errorMessage: error.message
        });
      });
    });
    return app.use(function(request, response) {
      response.status(404).send('Not found');
      return closeServer('Unknown path or verb');
    });
  });
};

exports.getContext = getContext = function(viewName) {
  if (viewName === 'success') {
    return Promise.props({
      dashboardUrl: resin.settings.get('dashboardUrl')
    });
  }
  return Promise.resolve({});
};
