// Generated by CoffeeScript 1.12.7
var Promise, QEMU_BIN_NAME, QEMU_VERSION, getQemuPath, hasQemu, installQemu, platformNeedsQemu;

Promise = require('bluebird');

exports.QEMU_VERSION = QEMU_VERSION = 'v2.5.50-resin-execve';

exports.QEMU_BIN_NAME = QEMU_BIN_NAME = 'qemu-execve';

exports.installQemuIfNeeded = Promise.method(function(emulated, logger) {
  if (!(emulated && platformNeedsQemu())) {
    return false;
  }
  return hasQemu().then(function(present) {
    if (!present) {
      logger.logInfo('Installing qemu for ARM emulation...');
      return installQemu();
    }
  })["return"](true);
});

exports.qemuPathInContext = function(context) {
  var binDir, binPath, path;
  path = require('path');
  binDir = path.join(context, '.resin');
  binPath = path.join(binDir, QEMU_BIN_NAME);
  return path.relative(context, binPath);
};

exports.copyQemu = function(context) {
  var binDir, binPath, fs, path;
  path = require('path');
  fs = require('mz/fs');
  binDir = path.join(context, '.resin');
  binPath = path.join(binDir, QEMU_BIN_NAME);
  return Promise.resolve(fs.mkdir(binDir))["catch"]({
    code: 'EEXIST'
  }, function() {}).then(function() {
    return getQemuPath();
  }).then(function(qemu) {
    return new Promise(function(resolve, reject) {
      var read, write;
      read = fs.createReadStream(qemu);
      write = fs.createWriteStream(binPath);
      return read.pipe(write).on('error', reject).on('finish', resolve);
    });
  }).then(function() {
    return fs.chmod(binPath, '755');
  }).then(function() {
    return path.relative(context, binPath);
  });
};

hasQemu = function() {
  var fs;
  fs = require('mz/fs');
  return getQemuPath().then(fs.stat)["return"](true).catchReturn(false);
};

getQemuPath = function() {
  var fs, path, resin;
  resin = require('resin-sdk').fromSharedOptions();
  path = require('path');
  fs = require('mz/fs');
  return resin.settings.get('binDirectory').then(function(binDir) {
    return Promise.resolve(fs.mkdir(binDir))["catch"]({
      code: 'EEXIST'
    }, function() {}).then(function() {
      return path.join(binDir, QEMU_BIN_NAME);
    });
  });
};

platformNeedsQemu = function() {
  var os;
  os = require('os');
  return os.platform() === 'linux';
};

installQemu = function() {
  var fs, request, zlib;
  request = require('request');
  fs = require('fs');
  zlib = require('zlib');
  return getQemuPath().then(function(qemuPath) {
    return new Promise(function(resolve, reject) {
      var installStream, qemuUrl;
      installStream = fs.createWriteStream(qemuPath);
      qemuUrl = "https://github.com/resin-io/qemu/releases/download/" + QEMU_VERSION + "/" + QEMU_BIN_NAME + ".gz";
      return request(qemuUrl).on('error', reject).pipe(zlib.createGunzip()).on('error', reject).pipe(installStream).on('error', reject).on('finish', resolve);
    });
  });
};
